package net.h34t.filemure.controller

import io.javalin.http.BadRequestResponse
import io.javalin.http.Context
import io.javalin.http.ForbiddenResponse
import net.h34t.filemure.*
import net.h34t.filemure.repository.SqliteRepository
import net.h34t.filemure.tpl.Frame
import net.h34t.filemure.tpl.NewDocumentForm
import java.io.File
import java.time.LocalDateTime
import java.time.format.DateTimeFormatter
import java.time.format.FormatStyle

class DocumentController(val modifiers: TemplateModifiers, val repository: SqliteRepository) {

    private val dtf = DateTimeFormatter.ofLocalizedDateTime(FormatStyle.MEDIUM, FormatStyle.SHORT)
    private val isoDtf = DateTimeFormatter.ofPattern("yyyy-MM-dd'T'HH:mm:ss")

    fun createDocumentForm(ctx: Context) {
        val session = ctx.requireSession()

        val fileIds = ctx.queryParams("file_id").map { it.toLong() }

        val limboFiles = repository.getFilesInLimbo(session.id)
        val limboFileIds = limboFiles.map { it.id }

        if (!fileIds.all { it in limboFileIds }) {
            throw ForbiddenResponse("Mismatched file ids.")
        }

        val selectedFiles = limboFiles.filter { it.id in fileIds }

        val title = "new document"
        val referenceDate = selectedFiles.asSequence().mapNotNull { DateGuesser.guess(it.filename) }.firstOrNull()
            ?: LocalDateTime.now()
        val tags = selectedFiles.map { File(it.filename).extension }.distinct().asSequence()
        val description = ""

        ctx.tempolin(
            Frame(
                modifiers = modifiers, title = "new document", isTarget = true, content = NewDocumentForm(
                    modifiers = modifiers,
                    title = title,
                    referenceDate = isoDtf.format(referenceDate),
                    tags = { tags.map { TagsBlock(it) } },
                    description = description,
                    files = {
                        selectedFiles
                            .map {
                                FilesBlock(
                                    id = it.id.toString(),
                                    filename = it.filename,
                                    contentType = it.contentType
                                )
                            }
                            .asSequence()
                    })
            )
        )
    }

    fun createDocument(ctx: Context) {
        val session = ctx.requireSession()

        val title = ctx.formParam("title")
            ?: throw BadRequestResponse("")
        val referenceDate = ctx.formParam("reference_date")?.let { LocalDateTime.parse(it, isoDtf) }
            ?: throw BadRequestResponse("")
        val tags = ctx.formParam("tags")?.split("\\s+")
            ?: emptyList()
        val description = ctx.formParam("description")
            ?: throw BadRequestResponse("")
        val fileIds = ctx.formParams("file_id").map { it.toLong() }

        val id = repository.addDocument(session.id, title, referenceDate, tags, description, fileIds)

        ctx.redirectPRG("/document/$id")
    }
}