package net.h34t.filemure.controller

import io.javalin.http.Context
import net.h34t.filemure.TemplateModifiers
import net.h34t.filemure.repository.SqliteRepository
import net.h34t.filemure.requireSession
import net.h34t.filemure.tempolin
import net.h34t.filemure.tpl.Frame
import net.h34t.filemure.tpl.Limbo
import java.time.format.DateTimeFormatter
import java.time.format.FormatStyle

class LimboController(val modifiers: TemplateModifiers, val repository: SqliteRepository) {

    private val dtf = DateTimeFormatter.ofLocalizedDateTime(FormatStyle.MEDIUM, FormatStyle.SHORT)
    private val isoDtf = DateTimeFormatter.ofPattern("yyyy-MM-dd'T'HH:mm:ss")

    fun formLimbo(ctx: Context) {
        val session = ctx.requireSession()

        val files = repository.getFilesInLimbo(accountId = session.id)

        ctx.tempolin(
            Frame(
                modifiers = modifiers, title = "Filemure Limbo", isTarget = true, content = Limbo(
                    modifiers = modifiers, limboFileCount = files.size.toString(), file = {
                        files.map { f ->
                            FileBlock(
                                id = f.id.toString(),
                                file = f.filename,
                                type = f.contentType,
                                size = f.size.toString(),
                                uploaded = f.created.format(dtf)
                            )
                        }.asSequence()
                    })
            )
        )
    }
}